==> YounifyConnectSDK-Android-Sample-0.9.15.tgz: app/src/main/java/tv/younify/sdk/connectapp/YounifyServicesViewModel.kt <==
package tv.younify.sdk.connectapp

import android.content.Context
import androidx.activity.result.ActivityResultRegistry
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import tv.younify.sdk.connect.Connect
import tv.younify.sdk.connect.FetchContentAllCategoriesCompleteCallback
import tv.younify.sdk.connect.FetchContentCategoryResult
import tv.younify.sdk.connect.FetchContentCategoryServiceCompleteCallback
import tv.younify.sdk.connect.FetchContentServiceErrorCallback
import tv.younify.sdk.connect.SearchContentServiceResult
import tv.younify.sdk.connect.StreamingCategory
import tv.younify.sdk.connect.StreamingContent
import tv.younify.sdk.connect.StreamingContentDetails
import tv.younify.sdk.connect.StreamingService

class YounifyServicesViewModel: ViewModel() {
    var services = MutableLiveData<MutableList<MutableLiveData<StreamingService>>>()

    suspend fun fetchServices() {
        val allServices = Connect.fetchServices()
        val mutableLiveList = mutableListOf<MutableLiveData<StreamingService>>()
        allServices.forEach {
            mutableLiveList.add(MutableLiveData(it))
        }

        services.value = mutableLiveList
    }

    suspend fun manageLinkedService(service: StreamingService, context: Context, registry: ActivityResultRegistry) {
        Connect.manageLinkedService(service, context, registry)
        updateService(service)
    }

    suspend fun linkService(service: StreamingService, context: Context, registry: ActivityResultRegistry): Boolean {
        val isLinked = Connect.linkService(service, context, registry)
        updateService(service)

        return isLinked
    }

    suspend fun unlinkService(service: StreamingService) {
        Connect.unlinkService(service)
        updateService(service)
    }

    suspend fun fetchCategories(): List<StreamingCategory> {
        return Connect.fetchCategories()
    }

    suspend fun fetchContent(
        categories: List<StreamingCategory>,
        services: List<StreamingService>,
        onServiceError: FetchContentServiceErrorCallback?,
        onCategoryServiceComplete: FetchContentCategoryServiceCompleteCallback?,
        onAllCategoriesComplete: FetchContentAllCategoriesCompleteCallback?
    ): List<FetchContentCategoryResult> {

        return Connect.fetchContent(
            categories,
            services,
            { service: StreamingService, error: Throwable? ->
                updateService(service)
                if (onServiceError != null)
                    onServiceError(service, error)
            },
            { category: StreamingCategory, service: StreamingService, content: List<StreamingContent>, error: Throwable? ->
                if (onCategoryServiceComplete != null)
                    onCategoryServiceComplete(category, service, content, error)
            },
            { result: List<FetchContentCategoryResult>, error: Throwable? ->
                if (onAllCategoriesComplete != null)
                    onAllCategoriesComplete(result, error)
            }
        )
    }

    suspend fun fetchContentChildren(content: StreamingContent): List<StreamingContent> {
        return Connect.fetchContentChildren(content)
    }

    suspend fun searchContent(services: List<StreamingService>, searchPhrase: String): List<SearchContentServiceResult> {
        return Connect.searchContent(services, searchPhrase, null, null)
    }

    suspend fun fetchContentDetails(content: StreamingContent): StreamingContentDetails? {
        return Connect.fetchContentDetails(content)
    }

    suspend fun fetchImage(url: String): ByteArray? {
        return Connect.fetchImage(url)
    }

    suspend fun refresh() {
        fetchServices()
    }

    private fun updateService(service: StreamingService) {
        val serviceLiveData = services.value?.firstOrNull { it.value?.name == service.name }
        if (serviceLiveData != null)
            serviceLiveData.value = service
    }
}

==> YounifyConnectSDK-Android-Sample-0.9.24.tgz: app/src/main/java/tv/younify/sdk/connectapp/YounifyServicesViewModel.kt <==
package tv.younify.sdk.connectapp

import android.content.Context
import androidx.activity.result.ActivityResultRegistry
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import tv.younify.sdk.connect.Connect
import tv.younify.sdk.connect.FetchContentAllCategoriesCompleteCallback
import tv.younify.sdk.connect.FetchContentCategoryResult
import tv.younify.sdk.connect.FetchContentCategoryServiceCompleteCallback
import tv.younify.sdk.connect.FetchContentServiceErrorCallback
import tv.younify.sdk.connect.SearchContentServiceResult
import tv.younify.sdk.connect.StreamingCategory
import tv.younify.sdk.connect.StreamingContent
import tv.younify.sdk.connect.StreamingContentDetails
import tv.younify.sdk.connect.StreamingService

class YounifyServicesViewModel: ViewModel() {
    var services = MutableLiveData<MutableList<MutableLiveData<StreamingService>>>()

    suspend fun fetchServices() {
        val allServices = Connect.fetchServices()
        val mutableLiveList = mutableListOf<MutableLiveData<StreamingService>>()
        allServices.forEach {
            mutableLiveList.add(MutableLiveData(it))
        }

        services.value = mutableLiveList
    }

    suspend fun manageLinkedService(service: StreamingService, context: Context, registry: ActivityResultRegistry) {
        try {
            Connect.manageLinkedService(service, context, registry)
        } finally {
            updateService(service)
        }
    }

    suspend fun linkService(service: StreamingService, context: Context, registry: ActivityResultRegistry): Boolean {

        var isLinked = false

        try {
            isLinked = Connect.linkService(service, context, registry)
        } finally {
            updateService(service)
        }

        return isLinked
    }

    suspend fun unlinkService(service: StreamingService) {
        try {
            Connect.unlinkService(service)
        } finally {
            updateService(service)
        }
    }

    suspend fun fetchCategories(): List<StreamingCategory> {
        return Connect.fetchCategories()
    }

    suspend fun fetchContent(
        categories: List<StreamingCategory>,
        services: List<StreamingService>,
        onServiceError: FetchContentServiceErrorCallback?,
        onCategoryServiceComplete: FetchContentCategoryServiceCompleteCallback?,
        onAllCategoriesComplete: FetchContentAllCategoriesCompleteCallback?
    ): List<FetchContentCategoryResult> {

        return Connect.fetchContent(
            categories,
            services,
            { service: StreamingService, error: Throwable? ->
                updateService(service)
                if (onServiceError != null)
                    onServiceError(service, error)
            },
            { category: StreamingCategory, service: StreamingService, content: List<StreamingContent>, error: Throwable? ->
                if (onCategoryServiceComplete != null)
                    onCategoryServiceComplete(category, service, content, error)
            },
            { result: List<FetchContentCategoryResult>, error: Throwable? ->
                if (onAllCategoriesComplete != null)
                    onAllCategoriesComplete(result, error)
            }
        )
    }

    suspend fun fetchContentChildren(content: StreamingContent): List<StreamingContent> {
        return Connect.fetchContentChildren(content)
    }

    suspend fun searchContent(services: List<StreamingService>, searchPhrase: String): List<SearchContentServiceResult> {
        return Connect.searchContent(services, searchPhrase, null, null)
    }

    suspend fun fetchContentDetails(content: StreamingContent): StreamingContentDetails? {
        return Connect.fetchContentDetails(content)
    }

    suspend fun fetchImage(url: String): ByteArray? {
        return Connect.fetchImage(url)
    }

    suspend fun refresh() {
        fetchServices()
    }

    private fun updateService(service: StreamingService) {
        val serviceLiveData = services.value?.firstOrNull { it.value?.name == service.name }
        if (serviceLiveData != null)
            serviceLiveData.value = service
    }
}
